@@ -31,9 +31,11 @@ CONTINUOUS_INTEGRATION = [
     travis_ci.ENVIRONMENT_VARIABLES,
 ]
 
+metadata_key = pytest.StashKey[dict]()
+
 
 def pytest_addhooks(pluginmanager):
-    from . import hooks
+    from pytest_metadata import hooks
 
     pluginmanager.add_hookspecs(hooks)
 
@@ -41,19 +43,20 @@ def pytest_addhooks(pluginmanager):
 @pytest.fixture(scope="session")
 def metadata(pytestconfig):
     """Provide test session metadata"""
-    return pytestconfig._metadata
+    return pytestconfig.stash[metadata_key]
 
 
 @pytest.fixture(scope="session")
 def include_metadata_in_junit_xml(metadata, pytestconfig, record_testsuite_property):
     """Provide test session metadata"""
-    metadata_ = pytestconfig._metadata
+    metadata_ = pytestconfig.stash[metadata_key]
     for name, value in metadata_.items():
         record_testsuite_property(name, value)
 
 
 def pytest_addoption(parser):
-    parser.addoption(
+    group = parser.getgroup("pytest-metadata")
+    group.addoption(
         "--metadata",
         action="append",
         default=[],
@@ -62,14 +65,14 @@ def pytest_addoption(parser):
         nargs=2,
         help="additional metadata.",
     )
-    parser.addoption(
+    group.addoption(
         "--metadata-from-json",
         action="store",
         default="{}",
         dest="metadata_from_json",
         help="additional metadata from a json string.",
     )
-    parser.addoption(
+    group.addoption(
         "--metadata-from-json-file",
         type=str,
         dest="metadata_from_json_file",
@@ -79,7 +82,7 @@ def pytest_addoption(parser):
 
 @pytest.hookimpl(tryfirst=True)
 def pytest_configure(config):
-    config._metadata = {
+    config.stash[metadata_key] = {
         "Python": platform.python_version(),
         "Platform": platform.platform(),
         "Packages": {
@@ -87,35 +90,34 @@ def pytest_configure(config):
             "pluggy": pluggy.__version__,
         },
     }
-    config._metadata.update({k: v for k, v in config.getoption("metadata")})
-    config._metadata.update(json.loads(config.getoption("metadata_from_json")))
+    config.stash[metadata_key].update({k: v for k, v in config.getoption("metadata")})
+    config.stash[metadata_key].update(
+        json.loads(config.getoption("metadata_from_json"))
+    )
     if config.getoption("metadata_from_json_file"):
         with open(config.getoption("metadata_from_json_file"), "r") as json_file:
-            config._metadata.update(json.load(json_file))
+            config.stash[metadata_key].update(json.load(json_file))
     plugins = dict()
     for plugin, dist in config.pluginmanager.list_plugin_distinfo():
         name, version = dist.project_name, dist.version
         if name.startswith("pytest-"):
             name = name[7:]
         plugins[name] = version
-    config._metadata["Plugins"] = plugins
+    config.stash[metadata_key]["Plugins"] = plugins
 
     for provider in CONTINUOUS_INTEGRATION:
-        [
-            config._metadata.update({var: os.environ.get(var)})
-            for var in provider
-            if os.environ.get(var)
-        ]
+        for var in provider:
+            if os.environ.get(var):
+                config.stash[metadata_key].update({var: os.environ.get(var)})
 
     if hasattr(config, "workeroutput"):
-        config.workeroutput["metadata"] = config._metadata
-
-    config.hook.pytest_metadata(metadata=config._metadata)
+        config.workeroutput["metadata"] = config.stash[metadata_key]
+    config.hook.pytest_metadata(metadata=config.stash[metadata_key], config=config)
 
 
 def pytest_report_header(config):
     if config.getoption("verbose") > 0:
-        return "metadata: {0}".format(config._metadata)
+        return "metadata: {0}".format(config.stash[metadata_key])
 
 
 @pytest.hookimpl(optionalhook=True)
@@ -123,4 +125,4 @@ def pytest_testnodedown(node):
     # note that any metadata from remote workers will be replaced with the
     # environment from the final worker to quit
     if hasattr(node, "workeroutput"):
-        node.config._metadata.update(node.workeroutput["metadata"])
+        node.config.stash[metadata_key].update(node.workeroutput["metadata"])
